-- Test payment_attachment
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(29);

set search_path to numerus, auth, public;

select has_table('payment_attachment');
select has_pk('payment_attachment');
select table_privs_are('payment_attachment', 'guest', array []::text[]);
select table_privs_are('payment_attachment', 'invoicer', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_attachment', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('payment_attachment', 'authenticator', array []::text[]);

select has_column('payment_attachment', 'payment_id');
select col_is_pk('payment_attachment', 'payment_id');
select col_is_fk('payment_attachment', 'payment_id');
select fk_ok('payment_attachment', 'payment_id', 'payment', 'payment_id');
select col_type_is('payment_attachment', 'payment_id', 'integer');
select col_not_null('payment_attachment', 'payment_id');
select col_hasnt_default('payment_attachment', 'payment_id');

select has_column('payment_attachment', 'original_filename');
select col_type_is('payment_attachment', 'original_filename', 'text');
select col_not_null('payment_attachment', 'original_filename');
select col_hasnt_default('payment_attachment', 'original_filename');

select has_column('payment_attachment', 'mime_type');
select col_type_is('payment_attachment', 'mime_type', 'text');
select col_not_null('payment_attachment', 'mime_type');
select col_hasnt_default('payment_attachment', 'mime_type');

select has_column('payment_attachment', 'content');
select col_type_is('payment_attachment', 'content', 'bytea');
select col_not_null('payment_attachment', 'content');
select col_hasnt_default('payment_attachment', 'content');


set client_min_messages to warning;
truncate payment_attachment cascade;
truncate payment cascade;
truncate company_user cascade;
truncate payment_method cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, role, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', 'invoicer', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', 'admin', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

set constraints "company_default_payment_method_id_fkey" deferred;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, country_code, currency_code, default_payment_method_id)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', 'ES', 'EUR', 222)
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', 'FR', 'USD', 444)
;

insert into payment_method (payment_method_id, company_id, name, instructions)
values (444, 4, 'cash', 'cash')
     , (222, 2, 'cash', 'cash')
;

set constraints "company_default_payment_method_id_fkey" immediate;

insert into company_user (company_id, user_id)
values (2, 1)
     , (4, 5)
;

insert into payment_account(payment_account_id, company_id, payment_account_type, name)
values (8, 2, 'other', 'Other 2')
     , (9, 4, 'other', 'Other 4')
;

insert into payment (payment_id, company_id, description, payment_account_id, payment_date, amount, currency_code)
values (13, 2, 'Payment 2', 8, '2011-01-11', 111, 'EUR')
     , (14, 4, 'Payment 4', 9, '2022-02-22', 222, 'EUR')
;

insert into payment_attachment (payment_id, original_filename, mime_type, content)
values (13, 'payment.txt', 'text/plain', convert_to('Payment 42', 'UTF8'))
     , (14, 'payment.html', 'text/html', convert_to('<html>Payment <em>42</em></html>', 'UTF8'))
;

prepare payment_attachment_data as
select payment_id, original_filename
from payment_attachment
order by payment_id, original_filename;

set role invoicer;
select is_empty('payment_attachment_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog');
select bag_eq(
	'payment_attachment_data',
	$$ values (13, 'payment.txt')
	$$,
	'Should only list payment attachmements of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog');
select bag_eq(
	'payment_attachment_data',
	$$ values (14, 'payment.html')
	$$,
	'Should only list payment attachmements of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie');
select throws_ok(
	'payment_attachment_data',
	'42501', 'permission denied for table payment_attachment',
	'Should not allow select to guest users'
);
reset role;


select *
from finish();

rollback;

